/********************************************************************************************************
		* PROGRAM      : menus
		* DATE - TIME  : jeudi 26 avril 2007 - 10h39
		* AUTHOR       : gilles bailly ( gilles.bailly@enst.fr )
			* FILENAME     : Flower16_3Menu.cpp
			* LICENSE      : GPL
			* COMMENTARY   :
********************************************************************************************************/

//FlowerGroupingMenu.cpp

#include <sstream>
#include <iostream>
#include <fstream>
#include <string>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#include <QPainter>
#include <QRectF>
#include <QColor>
#include <QEvent>
#include <QMouseEvent>


#include "FlowerGroupingMenu.h"
#include "FlowerGroupingSector.h"
#include "GMConfiguration.h"
#include "FlowerRecognizer.h"

using namespace std;



//======================
FlowerGroupingMenu::FlowerGroupingMenu(QWidget *parent):RadialMenu(parent), l0(7){
	
	stateClosed = new FlowerGroupingStateClosed(this);
	stateBegin = new FlowerGroupingStateBegin(this);
	stateTracing = new FlowerGroupingStateTracing(this);
	stateConfiguration = new FlowerGroupingStateConfiguration( this );
	stateCurrent = stateClosed;
	modeKey = false;
	configuration = new GMConfiguration(this);
	flowerRecognizer = new FlowerRecognizer();
	QString nameFile = ":conf/KppBaseFile.txt";
	
	flowerRecognizer->loadGesturesBase( nameFile );
	drawController = false;
}


//======================
FlowerGroupingMenu::~FlowerGroupingMenu(){
	delete stateClosed;
	delete stateBegin;
	
	delete stateTracing;
	delete stateConfiguration;
	
	delete configuration;
	delete flowerRecognizer;
}

//======================
void FlowerGroupingMenu::addFlowerGroupingSector(FlowerGroupingSector* _sector){
	
	int id = getNbSector();
	
	if(id > 16){
		cerr<<"deja 16 elements, sector ignore"<<endl;
		return;
	}
	_sector->setId(id);
	_sector->setParent(this);
}

//=====================
void FlowerGroupingMenu::addSpecificSector(FlowerGroupingSector* _sector, int id ){
	vItem.push_back( id );
	_sector->setId( id );
	_sector->setParent( this );
}

//====================
int FlowerGroupingMenu::getDirAndBranch( int id, int & branch){
	int res = id / 100;
	branch = id - 100 * res;
	return res;
}

//=====================
void FlowerGroupingMenu::defineConfiguration(int * _dir, int * _branch, int nb){
	configuration->defineConfiguration( _dir, _branch, nb);
}

//=====================
int FlowerGroupingMenu::getExpertGesture(){
	vector<QPointF > stroke = recognizer->getList();
	float* feature = new float[ 5 ];
	flowerRecognizer->getKPPFeature( stroke, feature );
	KPlusProche result; result.k = 5;
	flowerRecognizer->getProbForGesture(  feature,  result );
	
	int item = result.getItem();
	
	
	return item;
}




//=====================
int FlowerGroupingMenu::getNoviceGesture(){
	//cout<<__PRETTY_FUNCTION__<<endl;
	int item = -1;
	int nbItem = 16;
	float denim = (float)nbItem /2.;
	QPointF c(width()/2., height()/2.);
	QPointF tmp = recognizer->getEndOfStroke();
	if(tmp == MenuGestureRecognizer::pointNonValid){
		return -1;
	}
	
	/*
	QPoint tmp2;
	tmp2.setX((int)tmp.x());
	tmp2.setY((int)tmp.y());
	
	QObjectList list = children();
	for(int i =0; i < list.size(); i++){
		RadialSector* sector = qobject_cast<RadialSector *> (list[i]);
		if(sector){
			if(sector->itemContains(tmp2)){
				item = sector->getId();
			}
		}
	}
	*/
	
	if(item == -1){
		item = getExpertGesture();
		
	}
	
	
	return item;
	
	
	
	
	
	
	
}

//======================
//return 1, -1, 0, -999 si pas valide
int FlowerGroupingMenu::getKey(){
	int res = 0;
	if(key == 65 || key ==16777249){
		res = -1;
	}else if ( key == 69  || key ==16777248  || key ==16777251){
		res = 1;
	}
	
	return res;
}


//======================
void FlowerGroupingMenu::setKey(int _k){ key = _k; }

//=====================
void FlowerGroupingMenu::setModeKey(bool b){
	modeKey = b;
}

//======================
bool FlowerGroupingMenu::getModeKey(){ return modeKey; }

//======================
void FlowerGroupingMenu::initialize(){
	int maxW = 0;
	int maxH = 15;
	FlowerGroupingSector* item;
	QObjectList list = children();
	
	for(int i =0; i < list.size(); i++){
		item = qobject_cast<FlowerGroupingSector *> (list[i]);
		if(item){
			if(item->itemWidth() > maxW) maxW = item->itemWidth();
			if(item->itemHeight()> maxH) maxH = item->itemHeight();
		}
	}
	
	for(int i =0; i < list.size(); i++){
		item = qobject_cast<FlowerGroupingSector *> (list[i]);
		if(item){
			item->itemResize(maxW, maxH);
			item->initialize();
		}
	}
	if(glassPane) glassPane->raise();
}



//=================
StateMachine* FlowerGroupingMenu::getCurrentState(){ return stateCurrent; }
void FlowerGroupingMenu::setDefaultState(){ stateCurrent = stateClosed; }

//======================================
//======================================
