/********************************************************************************************************
		* PROGRAM      : menus
		* DATE - TIME  : samedi 28 avril 2007 - 12h52
		* AUTHOR       : gilles bailly ( gilles.bailly enst.fr )
			* FILENAME     : FlowerGroupingStateMachine.cpp
			* LICENSE      : GPL
			* COMMENTARY   :
********************************************************************************************************/
//FlowerGroupingStateMachine.cpp

#include <iostream>
#include <iostream>
#include <math.h>
#include "FlowerGroupingMenu.h"
#include "FlowerGroupingSector.h"
#include "FlowerGroupingStateMachine.h"
#include <QPainter>
#include <QRectF>
#include <QColor>
#include <QEvent>
#include <QMouseEvent>

using namespace std;
//=============================================================//
//==================== State Machine ==========================//
FlowerGroupingStateMachine::FlowerGroupingStateMachine(FlowerGroupingMenu* _menu):menu(_menu){ }

//=================
bool FlowerGroupingStateMachine::kPress(QObject * obj, QKeyEvent * ke){
	//menu->setKey( ke->key() );
	menu->drawController = true;
	menu->stateCurrent = menu->stateConfiguration;
	
	QObjectList list = menu->children();
	menu->timerFinish.stop();
	for(int i =0; i < list.size(); i++){
		FlowerGroupingSector* sector = qobject_cast<FlowerGroupingSector *> (list[i]);
		if(sector){
			sector->setItemDown(false);
			sector->setVisible( true );
		}
	}
	
	menu->show();
	menu->repaint();
	return true;
}


//=================
bool FlowerGroupingStateMachine::kRelease(QObject * obj, QKeyEvent * ke){
	//menu->setKey( ke->key() );
	QObjectList list = menu->children();
	menu->timerFinish.stop();
	
	for(int i =0; i < list.size(); i++){
		FlowerGroupingSector* sector = qobject_cast<FlowerGroupingSector *> (list[i]);
		if(sector){
			sector->setItemDown(false);
			sector->setVisible( false );
		}
	}
	
	menu->closeMenu();
	menu->stateCurrent = menu->stateClosed;
	FlowerGroupingSector::writePositionFile();
	menu->drawController = false;
	return true;
}

//=============================================================//
//===================== Closed ================================//
FlowerGroupingStateClosed::FlowerGroupingStateClosed(FlowerGroupingMenu* _menu):FlowerGroupingStateMachine(_menu){ }


//================
bool FlowerGroupingStateClosed::mPress( QObject *obj, QMouseEvent *e ){
	menu->setKey(-999);
	
	QFlags<Qt::KeyboardModifier> modifier = e->modifiers();
	if( modifier == Qt::ShiftModifier){
	
		menu->setKey(16777248);
	}else if (modifier == Qt::ControlModifier){
		
		menu->setKey(16777249);
	}else if ( modifier == Qt::AltModifier ){

		menu->setKey(16777251);
	}
	
	
	
	
	
	QObjectList list = menu->children();
	menu->timerFinish.stop();
	for(int i =0; i < list.size(); i++){
		FlowerGroupingSector* sector = qobject_cast<FlowerGroupingSector *> (list[i]);
		if(sector){
			sector->setItemDown(false);
			sector->setVisible(false);
		}
	}
	menu->show();
	menu->move( (int)(e->x() - menu->width() /2.),
			(int)(e->y() - menu->height() /2.)); //center
	//menu->openMenu(e->x(), e->y());
	menu->timerStart.start(menu->delayStart);
	
	menu->recognizer->startGesture(e->x() - menu->x(), e->y() - menu->y());
	menu->stateCurrent = menu->stateBegin;
	return true;
}

//================
bool FlowerGroupingStateClosed::mRelease( QObject *obj, QMouseEvent *e ){
	cerr<<__PRETTY_FUNCTION__<<endl;
	return true;
}

//================
bool FlowerGroupingStateClosed::mDrag( QObject *obj, QMouseEvent *e ){
	cerr<<__PRETTY_FUNCTION__<<endl;
	return true;
}


//============================================================//
//===================== Begin ================================//
FlowerGroupingStateBegin::FlowerGroupingStateBegin(FlowerGroupingMenu* _menu):FlowerGroupingStateMachine(_menu){ }

//================
bool FlowerGroupingStateBegin::mPress( QObject *obj, QMouseEvent *e ){
	cerr<<__PRETTY_FUNCTION__<<endl;
	return true;
}

//================
bool FlowerGroupingStateBegin::mRelease( QObject *obj, QMouseEvent *e ){
	menu->recognizer->endGesture(e->x() -menu->x(), e->y() -menu->y());
	menu->timerStart.stop();
	menu->timerFinish.stop();
	menu->stateCurrent = menu->stateClosed;
	menu->stop();
	menu->closeMenu();
	return true;
}

//================
bool FlowerGroupingStateBegin::mDrag( QObject *obj, QMouseEvent *e ){
	float X = (e->x()-menu->x()) - menu->width()/2;
	float Y = (e->y()-menu->y()) - menu->height()/2;
	float dist = sqrt(X*X + Y*Y);
	//cout<<"state machine mDrag "<<X<<endl;
	
	menu->recognizer->addPoint(e->x() - menu->x(),e->y() - menu->y());
	
	if(dist > menu->getCenterDist()){
		menu->stateCurrent = menu->stateTracing;
	}
	return true;
}


//============================================================//
//===================== Tracing ================================//
FlowerGroupingStateTracing::FlowerGroupingStateTracing(FlowerGroupingMenu* _menu):FlowerGroupingStateMachine(_menu){ }


//================
bool FlowerGroupingStateTracing::mPress( QObject *obj, QMouseEvent *e ){
	cerr<<__PRETTY_FUNCTION__<<endl;
	return true;
}

//================
bool FlowerGroupingStateTracing::mRelease( QObject *obj, QMouseEvent *e ){
	//menu->recognizer->addPoint(e->x() - menu->x(),e->y() - menu->y());
	//menu->recognizeGesture();
	//menu->recognizer->recognizeGesture();
	
	//menu->lastItemSelected = menu->curItemSelected;
	//menu->recognizer->endGesture(e->x() -menu->x(), e->y() -menu->y());
	
	if( menu->delayFinish ){
		menu->openMenu();
	}
	
	
	menu->clearTrace();
	menu->timerStart.stop();
	
	if( menu->delayFinish ){
		QObjectList children = menu->children();
		for(int i =0; i < children.size(); i++){
			FlowerGroupingSector* sector = qobject_cast<FlowerGroupingSector *> (children[i]);
			if(sector){
				if (sector->getId() != menu->curItemSelected){
					sector->setVisible(false);
				}
				
			}
		}
		
		
		
		menu->repaint();
		menu->show();
	}
	
	menu->timerFinish.start( menu->delayFinish );
	menu->stateCurrent = menu->stateClosed;
	menu->activeFinalItemSelected();
	
	return true;
}

//================
bool FlowerGroupingStateTracing::mDrag( QObject *obj, QMouseEvent *e ){
	float X = (e->x()-menu->x()) - menu->width()/2;
	float Y = (e->y()-menu->y()) - menu->height()/2;
	float dist = sqrt(X*X + Y*Y);
	
	if(menu->timerStart.isActive()){
		menu->timerStart.start(menu->delayStart);
	}else{
		
		
		if(dist < menu->getCenterDist()){
			menu->stateCurrent = menu->stateBegin;
			menu->abortGesture();
			menu->recognizeGesture();
			QObjectList children = menu->children();
			for(int i =0; i < children.size(); i++){
				FlowerGroupingSector* button = qobject_cast<FlowerGroupingSector *> (children[i]);
				if(button){
					if (button->getId() == menu->curItemSelected){
						button->setItemDown(true);
					}else{
						button->setItemDown(false);
					}
				}
				
			}
			return true;
		}
	}
	
	
	
	
	
	menu->recognizer->addPoint(e->x() - menu->x(),e->y() - menu->y());
	menu->recognizeGesture();
	
	QObjectList children = menu->children();
	for(int i =0; i < children.size(); i++){
		FlowerGroupingSector* button = qobject_cast<FlowerGroupingSector *> (children[i]);
		if(button){
			if (button->getId() == menu->curItemSelected){
				button->setItemDown(true);
			}else{
				button->setItemDown(false);
			}
		}
	}
	
	
	
	return true;
	
}



//=======================================
FlowerGroupingStateConfiguration::FlowerGroupingStateConfiguration( FlowerGroupingMenu* _menu ):FlowerGroupingStateMachine(_menu){
	sectorCur = 0;
}

//========================
bool containPoint( QPointF& p, float x, float y){
	return (::sqrt( ( p.x() - x ) * ( p.x() - x ) + ( p.y() - y ) * (p.y() - y) ) < 6 );
}

//==================================
bool FlowerGroupingStateConfiguration::mPress(QObject* obj, QMouseEvent* e ){
	float X = ( e->x() - menu->x() ) ;
	float Y = ( e->y() - menu->y() ) ;
	QPoint position( X, Y );
	
	QObjectList children = menu->children();
	for(int i =0; i < children.size(); i++){
		FlowerGroupingSector*sector = qobject_cast<FlowerGroupingSector *> (children[i]);
		if( sector && sectorCur == 0){
			if(  sector->itemContains( position ) ){
				sectorCur = sector;
				type = 0;
				xDiff = sector->itemX() - position.x();
				yDiff = sector->itemY() - position.y();
				
			}else if ( containPoint( FlowerGroupingSector::lineDepart[ sector->getId() ], X, Y ) ){
				sectorCur = sector;
				type = 1;
				xDiff = FlowerGroupingSector::lineDepart[ sector->getId() ].x() - position.x();
				yDiff = FlowerGroupingSector::lineDepart[ sector->getId() ].y() - position.y();
				
			}else if ( containPoint( FlowerGroupingSector::controler[ sector->getId() ], X, Y ) ){
				sectorCur = sector;
				type = 2;
				xDiff = FlowerGroupingSector::controler[ sector->getId() ].x() - position.x();
				yDiff = FlowerGroupingSector::controler[ sector->getId() ].y() - position.y();
				
			}else if ( containPoint( FlowerGroupingSector::lineFin[ sector->getId() ], X, Y ) ){
				sectorCur = sector;
				type = 3;
				xDiff = FlowerGroupingSector::lineFin[ sector->getId() ].x() - position.x();
				yDiff = FlowerGroupingSector::lineFin[ sector->getId() ].y() - position.y();
				
			}
			
		}
	}
	
	
	return true;
}

//==================================
bool FlowerGroupingStateConfiguration::mDrag(QObject* obj, QMouseEvent* e ){
	float X = (e->x() - menu->x()) ;
	float Y = (e->y() - menu->y()) ;
	if( sectorCur == 0 ) return true;
	
	if( type == 0) {
		QPointF p(  sectorCur->itemX() - ( X + xDiff ), sectorCur->itemY() - ( Y + yDiff ) ) ;
		sectorCur->moveItem( X + xDiff, Y + yDiff );
		FlowerGroupingSector::lineFin[ sectorCur->getId() ] -= p;
	}
	QPoint tmp( X + xDiff, Y + yDiff);
	
	
	if( type == 1 ) FlowerGroupingSector::lineDepart[ sectorCur->getId() ] = tmp;
	if( type == 2 ) FlowerGroupingSector::controler[ sectorCur->getId() ] = tmp;
	if( type == 3 ) FlowerGroupingSector::lineFin[ sectorCur->getId() ] = tmp;
	
	menu->repaint();
	return true;
}

bool FlowerGroupingStateConfiguration::mRelease(QObject* obj, QMouseEvent* e ){
	float X = (e->x() - menu->x());
	float Y = (e->y() - menu->y());
	if( sectorCur == 0 ) return true;
	
	QPoint p( X + xDiff, Y + yDiff );
	if( type == 0 ) FlowerGroupingSector::position[ sectorCur->getId() ] = p;
	sectorCur = 0;
	return true;
}











