/********************************************************************************************************
		* PROGRAM      : menus
		* DATE - TIME  : jeudi 26 avril 2007 - 10h18
		* AUTHOR       : gilles bailly ( gilles.bailly@enst.fr )
			* FILENAME     : RadialMenu.cpp
			* LICENSE      : GPL
			* COMMENTARY   :
********************************************************************************************************/
//RadialMenu.cpp


#include <iostream>
#include <QPainter>
#include <QRectF>
#include <QColor>
#include <QEvent>
#include <QMouseEvent>
#include <QPainterPath>
#include "RadialMenu.h"
#include "RadialSector.h"
#include "StateMachine.h"
#include "Delimiter.h"

using namespace std;

QObject* RadialMenu::mainWindow = 0;
RadialMenu* RadialMenu::menuForEvent =0;
RadialMenuData* RadialMenu::defaultRadialMenuData = new RadialMenuData();
string RadialMenu::MENU_PATH = ".";


//====================
RadialMenuData::RadialMenuData(){
	centerDist = 10;
	font = new QFont("Helvetica", 10);
	itemW = 50;
	itemH = 50;
}

//============================================
//============================================
RadialMenu::RadialMenu(QWidget* parent):QWidget(parent),delimiter(0){
	glassPane = 0;
	delimiter = new Delimiter(this);
	recognizer = new MenuGestureRecognizer();
	rdData = defaultRadialMenuData;
	delayStart = 400;
	delayFinish = 500;
	curItemSelected = -1;
	resize(1000, 1000);
	hide();
	setMouseTracking(true);
	traceInNoviceMode = true;
	//viewport()->setMouseTracking(true);
	
	timerFinish.setSingleShot(true);
	timerStart.setSingleShot(true);
	connect(&timerFinish, SIGNAL(timeout()), this, SLOT(timerCloseOut()));
	connect(&timerStart, SIGNAL(timeout()), this, SLOT(timerOpenOut()));
	
}

//====================
RadialMenu::~RadialMenu(){
	delete recognizer;
}


//=====================
QString RadialMenu::getNameOfChild(int _id){
	QObjectList list = children();
	for(int i =0; i < list.size(); i++){
		RadialSector* sector = qobject_cast<RadialSector *> (list[i]);
		if(sector ){
			if(sector->id == _id){
				
				return sector->text();
			}
			
		}
	}
	return "non valide";
}

//======================
void RadialMenu::setRoot(){
	
}

//======================
void RadialMenu::start(QObject* obj, QEvent* e){
	
	mainWindow = obj;
	if(!glassPane){
		glassPane =	new GlassPane(this);
		glassPane->setMenu(this);
		glassPane->raise();
		glassPane->resize(width(), height());
	}
	
	if(mainWindow){
		mainWindow->installEventFilter(this);
		menuForEvent = this;
		eventFilter(mainWindow, e);
		emit menuOpen();
	}else{
		cerr<<" pas de mainWindow dand RadialMenu"<<endl;
	}
	setRoot();
	
}

//=====================
void RadialMenu::addShortCutCommand( const ShortCutCommand& scc ){ shortCutCommand.push_back( scc );  }

//=====================
void RadialMenu::startShortCut( QKeyEvent* ke ){
	
	
	if(!glassPane){
		glassPane =	new GlassPane(this);
		glassPane->setMenu(this);
		glassPane->raise();
		glassPane->resize(width(), height());
	}
	
	if(mainWindow){
		menuForEvent = this;
		emit menuOpen();
	}else{
		cerr<<" pas de mainWindow dand RadialMenu"<<endl;
	}
	setRoot();
	curItemSelected = transformShortCut( ke );
	activeFinalItemSelected();
	stop();
	hide();
	ke->accept();
}

//=====================
int RadialMenu::transformShortCut( QKeyEvent *ke ){ return -1; }

//======================
void RadialMenu::stop(){
	if(glassPane) glassPane->clear();
	if(mainWindow){ mainWindow->removeEventFilter(this); }
	emit menuClose();
	curItemSelected = -1;
}

//=====================
void RadialMenu::menuReset(){
	hide();
	timerStart.stop();
	timerFinish.stop();
	setDefaultState();
	stop();
	
}

//=====================
void RadialMenu::setDelayStart( int _delay ){
	delayStart = _delay;
}

//======================
void RadialMenu::setDelayFinish( int _delay ){
	delayFinish = _delay;
}

//=====================
void RadialMenu::activeFinalItemSelected(){
	if(curItemSelected >-1){
		emit finalItemSelected(curItemSelected);
	}
}

//=====================
const MenuGestureRecognizer& RadialMenu::getRecognizer()const{
	return *recognizer;
}

//=====================
int RadialMenu::recognizeGesture(){
	int item = -1;
	if(isNoviceMode()){
		item = getNoviceGesture();
	}else{
		item = getExpertGesture();
	}
	
	if(item != curItemSelected){
	}
	curItemSelected = item;
	
	
	if(glassPane)
		glassPane->repaint();
	
	return item;
}

//=====================
void RadialMenu::setCenterDist(float _centerDist){ rdData->centerDist = _centerDist; }

//=====================
float RadialMenu::getCenterDist(){ return rdData->centerDist; }


//=====================
int RadialMenu::getNbSector(){
	int nb = 0;
	QObjectList list = children();
	for(int i=0; i< list.size(); i++){
		RadialSector* sector = qobject_cast<RadialSector *> (list[i]);
		if( sector )
			nb++;
	}
	return nb;
}

//=======================
int RadialMenu::depth()const{
	RadialSector* sector = qobject_cast<RadialSector *> ( parent() );
	if(sector){
		RadialMenu* menu = qobject_cast<RadialMenu *> ( sector->parent() );
		if(menu){
			return menu->depth() + 1;
		}
	}
	return 1;
}

//=======================
void RadialMenu::openMenu(){
	QObjectList list = children();
	timerFinish.stop();
	for(int i =0; i < list.size(); i++){
		RadialSector* sector = qobject_cast<RadialSector *> (list[i]);
		if(sector){
			//sector->setItemDown(false);
			sector->setVisible(true);
		}
	}
	show();
	
}

//======================
void RadialMenu::closeMenu(){
	
	hide();
	RadialSector* sector = qobject_cast<RadialSector *> ( parent() );
	if(sector){
		RadialMenu* menu = qobject_cast<RadialMenu *> ( sector->parent() );
		if(menu){
			return menu->closeMenu();
		}
	}
	stop();
}

//=======================
bool RadialMenu::isNoviceMode(){
	return !timerStart.isActive();
	/*
	bool res = true;
	Flower12Sector* item;
	QObjectList list = children();
	
	for(int i =0; i < list.size(); i++){
		item = qobject_cast<Flower12Sector *> (list[i]);
		if(item){
			res = res && item->isVisible();
		}
	}
	*/
}


//=====================
void RadialMenu::timerCloseOut(){ closeMenu(); }

//======================
void RadialMenu::timerOpenOut(){
	emit menuNoviceMode();
	openMenu();
}


//======================
void RadialMenu::setTraceInNoviceMode( bool b ){ traceInNoviceMode = b; }

//======================
QFont& RadialMenu::font(){ return *(rdData->font); }

//======================
void RadialMenu::setMenuFont( QFont f ){ *(rdData->font) = f; }

//=======================
bool RadialMenu::eventFilter( QObject *obj, QEvent *event ){
	//cout<<"event filter :"<<event->type()<<" depth "<<depth()<<endl;
	
	
	StateMachine* stateCurrent =  menuForEvent->getCurrentState();
	if(!stateCurrent){

		exit(0);
	}
	
	QMouseEvent* me;
	QKeyEvent* ke;
	switch( event->type() ){
		case QEvent::MouseButtonPress:
			me = dynamic_cast<QMouseEvent*>(event);
			tracing = true;
			if(glassPane) glassPane->clear();
			emit menuPress(me->x(), me->y());
			if( stateCurrent->mPress( obj, me ) )
					return true;
			//return false;
			break;
			
		case QEvent::MouseButtonRelease:
			me = dynamic_cast<QMouseEvent*>(event);
			emit menuRelease(me->x(), me->y());
			tracing = false;
			if( stateCurrent->mRelease( obj, me ) )
					return true;
			//return false;
			
			break;
			
		case QEvent::MouseMove:
			me = dynamic_cast<QMouseEvent*>(event);
			//cerr<<"move drag "<<endl;
			if(tracing){
				if(glassPane) glassPane->addPoint(QPointF(me->x() - x(), me->y() - y()) );
				emit menuDrag(me->x(), me->y());
				if( stateCurrent->mDrag( obj, me ) )
						return true;
			}else{
				if( stateCurrent->mMove( obj, me ) )
						return true;
			}
			
			break;
		case QEvent::KeyPress:
			ke =  dynamic_cast<QKeyEvent*>(event);
			ke->accept();
			if( stateCurrent->kPress( obj, ke ) )
					return true;
			break;
		case QEvent::KeyRelease:
			ke =  dynamic_cast<QKeyEvent*>(event);
			//cerr<<"key release ="<<ke->key()<<endl;
			ke->accept();
			if( stateCurrent->kRelease( obj, ke ) )
					return true;
			break;
			
		default:
			
			return false;
			break;
	}
	
	//return QObject::eventFilter( obj, event );
	return false;
}


//===========================
void RadialMenu::showSector(int num, bool novice){
	QObjectList myChildren = children();
	for(int i =0; i < myChildren.size(); i++){
		RadialSector* sector = qobject_cast<RadialSector *> (myChildren[i]);
		if(sector){
			if(sector->getId() == num){
				if(novice){
					sector->mainWidget->show();
				}else{
					sector->mainWidget->hide();
				}
				sector->show();
			}else{
				sector->hide();
			}
		}
	}
	
}

//===============================
void RadialMenu::clearTrace(){ glassPane->clear(); }

//================================
void RadialMenu::abortGesture( ){
	recognizer->abortGesture();
	clearTrace();
	repaint();
}


//================================
bool RadialMenu::contains(int x, int y){ return true; }

//===============================
RadialMenu* RadialMenu::getMenuEvent(int x, int y){
	if( contains(x, y) ) {
		return this;
	}else{
		return 0;
		//return getRoot();
		
	}
	
}

//===================================================================
//===================================================================
//===================================================================
GlassPane::GlassPane(QWidget* parent):QWidget(parent){
	move(0,0);
	resize(1000, 1000);
	setMouseTracking(true);
	trace.clear();
	//setAttribute(Qt::WA_NoBackground);
	//setAttribute(Qt::WA_NoSystemBackground);
}

//==========================
void GlassPane::addPoint(QPointF point){
	trace.push_back(point);
}

//===========================
void GlassPane::clear(){ trace.clear(); }

//===========================
void GlassPane::paintEvent(QPaintEvent* e){
	if( menu->isNoviceMode() && !menu->traceInNoviceMode ){ return; }
	
	QPainter p(this);
	p.setRenderHint( QPainter::Antialiasing );
	p.setBackgroundMode(Qt::TransparentMode);
	QPen pen;
	
	pen.setWidth(3);
	pen.setStyle(Qt::SolidLine);
	pen.setColor(Qt::red);
	p.setPen(pen);
	
	p.drawPoint((int)(width() /2.), (int)(height() / 2.));
	
	
	//const vector<QPointF>& trace = getMenu().getRecognizer().getList();
	if(trace.size()> 2){
		
		int iBegin = (int)trace.size() -30;
		if (iBegin <0) iBegin = 0;
		int lenght;
		if(trace.size()>30){
			lenght = 30;
		}else{
			lenght = trace.size();
		}
		
		int ind;
		for(int i =0; i<30; i++){
			ind = trace.size() - 1 -i;
			int transparency;
			if(i<10){
				transparency = 255;
			}else{
				transparency = -10 * i + 305;
			}
			if (ind > 0){
				pen.setColor(QColor(255,0,0, transparency ));
				p.setPen(pen);
				p.drawLine(trace[ind], trace[ind -1]);
			}else{
				return;
			}
		}
		
	}
}


