/********************************************************************************************************
		* PROGRAM      : menus
		* DATE - TIME  : samedi 28 avril 2007 - 12h21
		* AUTHOR       : gilles bailly ( gilles.bailly enst.fr )
			* FILENAME     : MenuGestureRecognizer.cpp
			* LICENSE      : GPL
			* COMMENTARY   :
********************************************************************************************************/
//MenuGestureRecognizer.cpp
#include <iostream>
#include "MenuGestureRecognizer.h"
#include <math.h>

using namespace std;


QPointF MenuGestureRecognizer::pointNonValid;

//==========
MenuGestureRecognizer::MenuGestureRecognizer(int min,
		int max,
		double _minMatch ){
			
	pointNonValid.setX(-9999);
	pointNonValid.setY(-9999);
	
	minMovement = min;
	maxMovement = max;
	minMatch = _minMatch;
}

//=============
MenuGestureRecognizer::~MenuGestureRecognizer(){}

//==============
void MenuGestureRecognizer::startGesture(int x, int y){
	posList.clear();
	posList.push_back(QPointF(x, y));
}

//==============
void MenuGestureRecognizer::addPoint(int x, int y){
	//cerr<<"lenght du stroke = "<<getLenght()<<" size = "<<getGestureSize()<<endl;
	int dx, dy;
	if(posList.size()==0){
		startGesture(x, y);
		return;
	}
	
	dx = x-(int)posList.back().x();
	dy = y-(int)posList.back().y();
	
	if( dx*dx+dy*dy >= minMovement ){
		posList.push_back(QPointF( x, y ) );
	}
}



//==============
void MenuGestureRecognizer::endGesture(int x, int y){
	if( x != (int)posList.back().x() || y != (int)posList.back().y() )
		posList.push_back( QPointF( x, y ) );
	
	if( posList.size() > 1 ){
		recognizeGesture();
	}else{
		//cas d'un clic
	}
	
	posList.clear();
}

//==============
void MenuGestureRecognizer::abortGesture(){
	posList.clear();
}

//=============
double MenuGestureRecognizer::angleAtFlower(double _x, double _y){
	double x = _x;
	double y =   _y; //y vers le bas //le moins a ete change
	double norm = x *x + y * y;
	if (norm == 0)
		return 0;
	
	x /= norm;
	y /=norm;
	double a;
	if(x != 0){
		a =	atan(y/x);
		if(x  < 0){  //peut etre le sens de >
			//a= -a;
			a = 3.14 - a;
		}else{
			//a =
			//a += 3.14 - a;
			a = -a;
			
		}
		//}
	}else{
		if(y > 0){ //le sens a ete change
			a = -3.14/2.;
		}else{
			a = 3.14 / 2.;
		}
	}
	if(a<0){a += 2 * 3.14; }
	
	return a;
}

//=============
double MenuGestureRecognizer::angleAt(double _x, double _y){
	double x = _x;
	double y =  _y; //y vers le bas //le moins a ete change
	double norm = x *x + y * y;
	if (norm == 0)
		return 0;
	
	x /= norm;
	y /=norm;
	double a;
	if(x != 0){
		a =	atan(y/x);
		if(x  < 0){  //peut etre le sens de >
			a = 3.14 - a;
		}else{
			//a =
			//a += 3.14;
			a = -a;
			
		}
		//}
	}else{
		if(y > 0){ //le sens a ete change
			a = -3.14/2.;
		}else{
			a = 3.14 / 2.;
		}
	}
	if(a<0){a += 2 * 3.14; }
	
	return a;
}

//==============
int MenuGestureRecognizer::getGestureSize(){
	return (int)posList.size();
}

//==============
QPointF MenuGestureRecognizer::getOriginOfStroke(){
	int iBegin = (int)posList.size() - 30;
	if (iBegin  <0) iBegin = 0;
	return posList[iBegin];
}

//==============
QPointF MenuGestureRecognizer::getEndOfStroke(){
	if(posList.size()>0){
		return posList.back();
	}else{
		QPointF res(-9999,-9999);
		return res;
	}
}


//================
int MenuGestureRecognizer::recognize64GestureL(float debut, float fin){
	float lenght = (float) getLenght();
	int ld = (int) ( lenght * debut );
	int lf = (int) (lenght * fin );
	
	int iDebut = -1;
	int iFin = -1;
	int l =0;
	int dx, dy;
	
	if( posList.size() < 2){ return-1; }
	
	for(unsigned int i = 0; i< posList.size() - 1; i++ ){
		dx = (int) ( posList[i+1].x() - posList[i].x() );
		dy = (int) ( posList[i+1].y() - posList[i].y() );
		l += (int) ::sqrt( dx * dx + dy * dy );
		
		if( l > ld && iDebut == -1){ iDebut = i; }
		if (l > lf && iFin == -1){ iFin = i + 1; }
	}
	if (iFin == -1){ iFin = posList.size() ; }
	return recognize64Gesture(iDebut, iFin);
}

//==================
int MenuGestureRecognizer::getNbPoints(){
	return posList.size();
}

//==================
int MenuGestureRecognizer::recognize64Gesture(int _debut, int _fin){
	int debut = _debut;
	int fin = _fin;
	int res = -1;
	if(!posList.size()> 2)
		return res;
	
	
	if( debut< 0){ debut = 0; }
	if(debut > (int)posList.size() -2){ debut = (int)posList.size() -2; }
	
	if( fin < 1){ fin = 1; }
	if(fin > (int)posList.size() -1){ fin = (int)posList.size() -1; }
	
	
	QPointF v = posList[fin] - posList[debut];
	
	
	double a = angleAt(v.x(), v.y());
	for( unsigned int i= 0; i<64; i+= 1){
		if( a>= 3.14 * (float)( 2* i + 1) / 64. &&  a< 3.14 * (float) (2*i + 3) / 64.){
			return i + 1;
		}
	}
	return 0;
	
	
}

//================
Direction MenuGestureRecognizer::recognize8GestureL(float debut, float fin){
	float lenght = (float) getLenght();
	int ld = (int) ( lenght * debut );
	int lf = (int) (lenght * fin );
	
	int iDebut = -1;
	int iFin = -1;
	int l =0;
	int dx, dy;
	
	if( posList.size() < 2){ return NoMatch; }
	
	for(unsigned int i = 0; i< posList.size() - 1; i++ ){
		dx = (int) ( posList[i+1].x() - posList[i].x() );
		dy = (int) ( posList[i+1].y() - posList[i].y() );
		l += (int) ::sqrt( dx * dx + dy * dy );
		
		if( l > ld && iDebut == -1){ iDebut = i; }
		if (l > lf && iFin == -1){ iFin = i + 1; }
	}
	if (iFin == -1){ iFin = posList.size() ; }
	return recognize8Gesture(iDebut, iFin);
}


//================
Direction MenuGestureRecognizer::recognize8Gesture(int _debut, int _fin){
	int debut = _debut;
	int fin = _fin;
	Direction d = NoMatch;
	if(!posList.size()> 2)
		return d;
	
	//int iBegin = posList.size() - 30;
	//if (iBegin < 0) iBegin = 0;
	
	if( debut< 0){ debut = 0; }
	if(debut > (int)posList.size() -2){ debut = (int)posList.size() -2; }
	
	if( fin < 1){ fin = 1; }
	if(fin > (int)posList.size() -1){ fin = (int)posList.size() -1; }
	
	
	QPointF v = posList[fin] - posList[debut];
	
	double a = angleAt(v.x(), v.y());
	
	if(a>= 0 && a< 3.14 / 8.){
		//cout<< "right"<<endl;
		d = Right;
	}else if (a >= 3.14/8. && a< 3.14 * 3. /8.){
		d = UpRight; //cout<<"upRight"<<endl;
	}else if (a >= 3.14 * 3. /8. && a< 3.14 * 5. /8.){
		d = Up; //cout<<"up"<<endl;
	}else if (a >= 3.14 * 5. /8. && a< 3.14 * 7./8.){
		d = UpLeft; //cout<<"upLeft"<<endl;
	}else if (a >= 3.14 * 7. /8. && a< 3.14 * 9./8.){
		d = Left; //cout<<"left"<<endl;
	}else if (a >= 3.14 * 9. / 8. && a< 3.14 * 11. /8){
		d = DownLeft; //cout<<"downLeft"<<endl;
	}else if (a >= 3.14 * 11./8. && a< 3.14 * 13./8.){
		d = Down; //cout<<"down"<<endl;
	}else if (a >= 3.14 * 13./8. && a< 3.14 * 15./8.){
		d = DownRight; //cout<<"downRight"<<endl;
	}else if (a >= 3.14 * 15./8. && a<= 3.14 * 16/8){
		d = Right; //cout<<"right"<<endl;
	}else{
		cout<<"pas d'angle correct "<< a <<endl;
	}
	return d;
	
}














//================
Direction MenuGestureRecognizer::recognize12GestureL(float debut, float fin){
	float lenght = (float) getLenght();
	int ld = (int) ( lenght * debut );
	int lf = (int) (lenght * fin );
	
	int iDebut = -1;
	int iFin = -1;
	int l =0;
	int dx, dy;
	
	if( posList.size() < 2){ return NoMatch; }
	
	for(unsigned int i = 0; i< posList.size() - 1; i++ ){
		dx = (int) ( posList[i+1].x() - posList[i].x() );
		dy = (int) ( posList[i+1].y() - posList[i].y() );
		l += (int) ::sqrt( dx * dx + dy * dy );
		
		if( l > ld && iDebut == -1){ iDebut = i; }
		if (l > lf && iFin == -1){ iFin = i + 1; }
	}
	if (iFin == -1){ iFin = posList.size() ; }
	return recognize12Gesture(iDebut, iFin);
}


//================
Direction MenuGestureRecognizer::recognize12Gesture(int _debut, int _fin){
	int debut = _debut;
	int fin = _fin;
	Direction d = NoMatch;
	if(!posList.size()> 2)
		return d;
	
	//int iBegin = posList.size() - 30;
	//if (iBegin < 0) iBegin = 0;
	
	if( debut< 0){ debut = 0; }
	if(debut > (int)posList.size() -2){ debut = (int)posList.size() -2; }
	
	if( fin < 1){ fin = 1; }
	if(fin > (int)posList.size() -1){ fin = (int)posList.size() -1; }
	
	
	QPointF v = posList[fin] - posList[debut];
	
	double a = angleAt(v.x(), v.y());
	
	if(a>= 0 && a< 3.14 / 12.){
		//cout<< "right"<<endl;
		d = Right;
	}else if (a >= 3.14/12. && a< 3.14 * 3. /12.){
		d = RightUp; //cout<<"upRight"<<endl;
	}else if (a >= 3.14 * 3. /12. && a< 3.14 * 5. /12.){
		d = UpRight; //cout<<"up"<<endl;
	}else if (a >= 3.14 * 5. /12. && a< 3.14 * 7./12.){
		d = Up; //cout<<"upLeft"<<endl;
	}else if (a >= 3.14 * 7. /12. && a< 3.14 * 9./12.){
		d = UpLeft; //cout<<"left"<<endl;
	}else if (a >= 3.14 * 9. / 12. && a< 3.14 * 11. /12){
		d = LeftUp; //cout<<"downLeft"<<endl;
	}else if (a >= 3.14 * 11./12. && a< 3.14 * 13./12.){
		d = Left; //cout<<"down"<<endl;
	}else if (a >= 3.14 * 13./12. && a< 3.14 * 15./12.){
		d = LeftDown; //cout<<"downRight"<<endl;
	}else if (a >= 3.14 * 15./12. && a< 3.14 * 17. /12.){
		d = DownLeft; //cout<<"upRight"<<endl;
	}else if (a >= 3.14 * 17. /12. && a< 3.14 * 19. /12.){
		d = Down; //cout<<"up"<<endl;
	}else if (a >= 3.14 * 19. /12. && a< 3.14 * 21./12.){
		d = DownRight; //cout<<"upLeft"<<endl;
	}else if (a >= 3.14 * 21. /12. && a< 3.14 * 23./12.){
		d = RightDown; //cout<<"left"<<endl;
	}else if (a >= 3.14 * 23./12. && a<= 3.14 * 24./12.){
		d = Right; //cout<<"right"<<endl;
	}else{
		cout<<"pas d'angle correct "<< a <<endl;
	}
	return d;
	
}







//==========================
Direction MenuGestureRecognizer::recognize4Gesture(int debut, int fin){
	Direction d = NoMatch;
	if(!posList.size()> 2)
		return d;
	
	int iBegin = posList.size() - 30;
	if (iBegin < 0) iBegin = 0;
	
	if( debut< 0){ debut = 0; }
	if((unsigned int)debut > posList.size() -2){ debut = (int)posList.size() -2; }
	
	if( fin < 1){ fin = 1; }
	if( (unsigned int)fin > posList.size() -1){ fin = (int)posList.size() -1; }
	
	
	QPointF v = posList[fin] - posList[debut];
	
	double a = angleAt(v.x(), v.y());
	
	if(a>= 0 && a< 3.14 * 2. / 8.){
		//cout<< "right"<<endl;
		d = Right;
	}else if (a >= 3.14 * 2. /8. && a< 3.14 * 6. /8.){
		d = Up; //cout<<"up"<<endl;
	}else if (a >= 3.14 * 6. /8. && a< 3.14 * 10./8.){
		d = Left; //cout<<"upLeft"<<endl;
	}else if (a >= 3.14 * 10. / 8. && a< 3.14 * 14. /8){
		d = Down; //cout<<"down"<<endl;
	}else if (a >= 3.14 * 14./8. && a< 3.14 * 16./8.){
		d = Right; //cout<<"right"<<endl;
	}else{
		cout<<"pas d'angle correct "<< a <<endl;
	}
	return d;
	
}

//==============
int MenuGestureRecognizer::getLenght(){
	int res = 0;
	if( posList.size() <2){
		return 0;
	}
	
	int dx, dy;
	for(unsigned int i =0; i < posList.size() - 1; i++){
		dx = (int) ( posList[i+1].x() - posList[i].x() );
		dy = (int) ( posList[i+1].y() - posList[i].y() );
		res += (int) ::sqrt( dx * dx + dy * dy );
		
	}
	return res;
}


//==============
Direction MenuGestureRecognizer::recognizeGesture(){
	return recognize8Gesture(posList.size() -30, posList.size());
	
}
